/*
 * Copyright (C) 2019-2022 HERE Europe B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 * License-Filename: LICENSE
 */

package com.example.heresdkandroidref.features;

import android.annotation.SuppressLint;
import android.content.Context;
import android.os.AsyncTask;
import android.util.Log;
import android.widget.SeekBar;
import android.widget.TextView;

import androidx.annotation.NonNull;
import androidx.appcompat.app.AlertDialog;
import androidx.core.content.ContextCompat;

import com.example.heresdkandroidref.R;
import com.example.heresdkandroidref.events.GetDetailsEvent;
import com.example.heresdkandroidref.models.TrafficLineColor;
import com.example.heresdkandroidref.utils.LoadingDialog;
import com.example.heresdkandroidref.utils.Utils;
import com.here.sdk.animation.AnimationListener;
import com.here.sdk.animation.AnimationState;
import com.here.sdk.core.Color;
import com.here.sdk.core.GeoCoordinates;
import com.here.sdk.core.GeoCoordinatesUpdate;
import com.here.sdk.core.GeoOrientation;
import com.here.sdk.core.GeoOrientationUpdate;
import com.here.sdk.core.GeoPolyline;
import com.here.sdk.core.Location;
import com.here.sdk.core.errors.InstantiationErrorException;
import com.here.sdk.gestures.GestureState;
import com.here.sdk.mapview.LineCap;
import com.here.sdk.mapview.MapCameraAnimation;
import com.here.sdk.mapview.MapCameraAnimationFactory;
import com.here.sdk.mapview.MapImage;
import com.here.sdk.mapview.MapImageFactory;
import com.here.sdk.mapview.MapMarker;
import com.here.sdk.mapview.MapMeasure;
import com.here.sdk.mapview.MapMeasureDependentRenderSize;
import com.here.sdk.mapview.MapPolyline;
import com.here.sdk.mapview.MapView;
import com.here.sdk.mapview.RenderSize;
import com.here.sdk.navigation.NavigableLocation;
import com.here.sdk.navigation.RouteProgress;
import com.here.sdk.navigation.SectionProgress;
import com.here.sdk.navigation.VisualNavigator;
import com.here.sdk.routing.Route;
import com.here.sdk.routing.Section;
import com.here.sdk.routing.Waypoint;
import com.here.time.Duration;

import org.greenrobot.eventbus.EventBus;

import java.util.ArrayList;
import java.util.List;
import java.util.Locale;

// An app that allows to calculate a route and start navigation, using either platform positioning or
// simulated locations.
public class App implements Navigation.NavigationListener {

    public static final String TAG = App.class.getSimpleName();
    public static final GeoCoordinates DEFAULT_MAP_CENTER = new GeoCoordinates(52.520798, 13.409408);
    public static final int DEFAULT_DISTANCE_IN_METERS = 1000 * 2;

    private final Context context;
    private final MapView mapView;
    private final List<MapMarker> mapMarkerList = new ArrayList<>();
    private final List<MapPolyline> mapPolylines = new ArrayList<>();
    private ArrayList<MapPolyline> trafficPolylines = new ArrayList<>();
    //    private final List<TrafficLineColor> trafficLineColors = new ArrayList<>();
//    private final List<MapPolyline> mapTravelledPolylines = new ArrayList<>();
    private final LoadingDialog dialog;
    private Waypoint startWaypoint;
    private Waypoint destinationWaypoint;
    private boolean isLongpressDestination;
    private final RouteCalculator routeCalculator;
    private final Navigation navigationExample;
    private final TextView messageView;
    private Boolean isTrafficEnabled = true;
    public Boolean isSimulated = true;
    public Boolean fromShare = false;
    private String selectedOption = "";
    private final List<Waypoint> waypointsList;
    public GeoOrientation geoOrientation = null;
    public double zoomLevel = 0;
    private final SeekBar routeProgressSeekbar;

    public App(Context context, MapView mapView, TextView messageView, SeekBar routeProgressSeekbar, List<Waypoint> waypointsList) {
        this.context = context;
        this.mapView = mapView;
        this.messageView = messageView;
        this.routeProgressSeekbar = routeProgressSeekbar;
        this.waypointsList = waypointsList;
        MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, DEFAULT_DISTANCE_IN_METERS);
        this.mapView.getCamera().lookAt(DEFAULT_MAP_CENTER, mapMeasureZoom);

        routeCalculator = new RouteCalculator(context);

        navigationExample = new Navigation(context, mapView, messageView, this);

        setLongPressGestureHandler();

        messageView.setText("Long press to set a destination or use a random one.");

        this.dialog = new LoadingDialog(context);
    }

    public void startLocationProvider() {
        navigationExample.startLocationProvider();
    }

    // Calculate a route and start navigation using a location simulator.
    // Start is map center and destination location is set random within viewport,
    // unless a destination is set via long press.
    public void addRouteSimulatedLocation(GeoCoordinates start, GeoCoordinates destination, String selectedOption, boolean isTrafficOn) {
        this.selectedOption = selectedOption;
        startWaypoint = new Waypoint(start);
        destinationWaypoint = new Waypoint(destination);
        isTrafficEnabled = isTrafficOn;
        calculateRoute(isSimulated, selectedOption, start, true);
    }

    // Calculate a route and start navigation using locations from device.
    // Start is current location and destination is set random within viewport,
    // unless a destination is set via long press.
    public void addRouteDeviceLocation(GeoCoordinates start, GeoCoordinates destination, String selectedOption, boolean isTrafficOn) {
        this.selectedOption = selectedOption;
        //Location location = navigationExample.getLastKnownLocation();
        //if(startWaypoint==null)
        //startWaypoint = new Waypoint(location.coordinates);
        //If a driver is moving, the bearing value can help to improve the route calculation.
        //startWaypoint.headingInDegrees = location.bearingInDegrees;
        //mapView.getCamera().lookAt(location.coordinates);
        destinationWaypoint = new Waypoint(destination);
        this.isTrafficEnabled = isTrafficOn;
        calculateRoute(isSimulated, selectedOption, start,true);
    }

    public void clearMapButtonPressed() {
        clearMap();
        isLongpressDestination = false;
    }

    public void toggleTrackingButtonOnClicked() {
        if (!isSimulated && navigationExample.getLastKnownLocation() != null) {
            MapMeasure mapMeasureZoom;
            if (zoomLevel == 0) {
                mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, 75);
            } else {
                mapMeasureZoom = new MapMeasure(MapMeasure.Kind.ZOOM_LEVEL, zoomLevel);
            }
            GeoOrientationUpdate orientationUpdate;
            if (geoOrientation == null) {
                orientationUpdate = new GeoOrientationUpdate(null, 70.0);
            } else {
                orientationUpdate = new GeoOrientationUpdate(geoOrientation.bearing, geoOrientation.tilt);
            }
            mapView.getCamera().lookAt(navigationExample.getLastKnownLocation().coordinates, orientationUpdate, mapMeasureZoom);
        }
        geoOrientation = null;
        // By default, this is enabled.
        navigationExample.startCameraTracking();
    }

    public void toggleTrackingButtonOffClicked() {
        navigationExample.stopCameraTracking();
    }

    private void calculateRoute(boolean isSimulated, String selectedOption, GeoCoordinates start, boolean firstTime) {
        LoadingDialog.showDialog(dialog);
        clearMap();

        if (!determineRouteWaypoints(start, isSimulated)) {
            return;
        }
        if (firstTime && start.distanceTo(startWaypoint.coordinates)<30 && isSimulated) {
            EventBus.getDefault().post(new GetDetailsEvent(new ArrayList<>()));
        } else {
            // Calculates a car route.
            routeCalculator.calculateRoute(startWaypoint, waypointsList, destinationWaypoint, selectedOption, (routingError, routes) -> {
                if (routingError == null) {
//                Route route = routes.get(0);
//                showRouteOnMap(route, isTrafficOn);
//                showRouteDetails(route, isSimulated);
                    ArrayList<Route> routesList = new ArrayList<>(routes != null ? routes : new ArrayList<>());
                    EventBus.getDefault().post(new GetDetailsEvent(routesList));
                } else {
                    showDialog("Error while calculating a route:", routingError.toString());
                    LoadingDialog.dismissDialog(dialog);
                }
            });
        }
    }

    private void calculateRouteRefresh(boolean isSimulated, String selectedOption, boolean isTrafficOn) {
        clearRoute();
        clearWaypointMapMarker();
        routeCalculator.calculateRoute(startWaypoint, waypointsList, destinationWaypoint, selectedOption, (routingError, routes) -> {
            if (routingError == null) {
                Route route = routes.get(0);
                Log.e("calculateRouteRefresh: ", "route updated" + route.getGeometry().vertices.get(0).latitude);
//                navigationExample.refreshRoute(route);
//                showRouteOnMap(route);
                //save route for saving in GPX
                routeCalculator.setCalculatedRoute(route);
            } else {
                showDialog("Error while calculating a route:", routingError.toString());
            }
        });
    }

    private boolean determineRouteWaypoints(GeoCoordinates start, boolean isSimulated) {
        if (!isSimulated && navigationExample.getLastKnownLocation() == null) {
            showDialog("Error", "No location found.");
            return false;
        }

        if (isSimulated) {
            startWaypoint = new Waypoint(start);
        } else {
            Location location = navigationExample.getLastKnownLocation();
//            if(startWaypoint==null)
            startWaypoint = new Waypoint(location.coordinates);
            // If a driver is moving, the bearing value can help to improve the route calculation.
            startWaypoint.headingInDegrees = location.bearingInDegrees;
//            mapView.getCamera().lookAt(location.coordinates);
        }
//
        if (!isLongpressDestination) {
            if (destinationWaypoint == null)
                destinationWaypoint = new Waypoint(createRandomGeoCoordinatesAroundMapCenter());
        }

        // Add circles to indicate start and destination of route.
        addCircleMapMarker(startWaypoint.coordinates, R.drawable.green_dot);
        addCircleMapMarker(destinationWaypoint.coordinates, R.drawable.green_dot);

        return true;
    }

    public void showRouteDetails(Route route, boolean isSimulated) {
        long estimatedTravelTimeInSeconds = route.getDuration().getSeconds();
        int lengthInMeters = route.getLengthInMeters();

        String routeDetails =
                "Travel Time: " + formatTime(estimatedTravelTimeInSeconds)
                        + ", Length: " + formatLength(lengthInMeters);
        navigationExample.startNavigation(route, isSimulated);
//        showStartNavigationDialog("Route Details", routeDetails, route, isSimulated);
    }

    private String formatTime(long sec) {
        int hours = (int) (sec / 3600);
        int minutes = (int) ((sec % 3600) / 60);

        return String.format(Locale.getDefault(), "%02d:%02d", hours, minutes);
    }

    private String formatLength(int meters) {
        int kilometers = meters / 1000;
        int remainingMeters = meters % 1000;

        return String.format(Locale.getDefault(), "%02d.%02d km", kilometers, remainingMeters);
    }

    @SuppressLint("ResourceAsColor")
    public void showRouteOnMap(Route route, boolean isTrafficOn) {
        // Show route as polyline.
        GeoPolyline routeGeoPolyline = route.getGeometry();
        float widthInPixels = 40;
        MapPolyline routeMapPolyline = null; // RGBA
        try {
            routeMapPolyline = new MapPolyline(routeGeoPolyline,
                    new MapPolyline.SolidRepresentation(new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, widthInPixels), Color.valueOf(ContextCompat.getColor(context, R.color.routeColor)), LineCap.ROUND));
        } catch (MapPolyline.Representation.InstantiationException |
                 MapMeasureDependentRenderSize.InstantiationException e) {
            e.printStackTrace();
        }
        mapView.getMapScene().addMapPolyline(routeMapPolyline);
        // The travelled part of the route should appear invisible to achieve a "route eat-up" effect.
        //routeMapPolyline.setProgressColor(Color.valueOf(android.R.color.transparent)); // Clear color.

        mapPolylines.add(routeMapPolyline);
        routeProgressSeekbar.setMax(route.getSections().get(0).getSpans().size());
        //update traffic lines
        drawTrafficPolylineForRoute(route, isTrafficOn);

    }

    // Call this continuously when you get a new RouteProgress event.
    public void updateRouteProgress(Route route, RouteProgress routeProgress) {
        List<SectionProgress> sectionProgressList = routeProgress.sectionProgress;
        // sectionProgressList is guaranteed to be non-empty.
        SectionProgress lastSectionProgress = sectionProgressList.get(sectionProgressList.size() - 1);
//        double progressOnRoute = 1 - (double) lastSectionProgress.remainingDistanceInMeters / route.getLengthInMeters();
//        if (!mapPolylines.isEmpty() && mapPolylines.get(0) != null) {
//            mapPolylines.get(0).setProgress(progressOnRoute);
//        }
        Log.e(TAG, "Section list size: " + sectionProgressList.size() + ", Last Section: " + sectionProgressList.indexOf(lastSectionProgress));
        routeProgressSeekbar.setProgress(routeProgressSeekbar.getProgress() + 1);
//            routeProgressSeekbar.setOnSeekBarChangeListener(new SeekBar.OnSeekBarChangeListener() {
//                @Override
//                public void onProgressChanged(SeekBar seekBar, int i, boolean b) {
//                    Log.e(TAG, "Progress: "+i);
//                }
//
//                @Override
//                public void onStartTrackingTouch(SeekBar seekBar) {
//
//                }
//
//                @Override
//                public void onStopTrackingTouch(SeekBar seekBar) {
//
//                }
//            });
    }


//    private void showRouteOnMapRefresh(GeoCoordinates currentCoordinates, VisualNavigator visualNavigator) {
//        if(visualNavigator.getRoute()!=null) {
//            new Handler(Looper.myLooper()).postDelayed(
//                    () -> {
//                        if (mapTravelledPolylines.isEmpty()) {
//                            // Show route as polyline.
//                            try {
//                                List<GeoCoordinates> coords = new ArrayList<>();
//                                coords.add(visualNavigator.getRoute() != null ? visualNavigator.getRoute().getGeometry().vertices.get(0) : currentCoordinates);
//                                coords.add(currentCoordinates);
//                                GeoPolyline routeTravelledGeoPolyline = new GeoPolyline(coords);
//                                float widthInPixels2 = 45;
//                                MapPolyline routeTravelledMapPolyline = new MapPolyline(routeTravelledGeoPolyline, widthInPixels2, Color.valueOf(ContextCompat.getColor(context, android.R.color.darker_gray)));
//                                mapView.getMapScene().addMapPolyline(routeTravelledMapPolyline);
//                                mapTravelledPolylines.add(routeTravelledMapPolyline);
//                            } catch (InstantiationErrorException e) {
//                                e.printStackTrace();
//                            }
//                        } else {
//                            GeoPolyline geoPolyline = mapTravelledPolylines.get(0).getGeometry();
//                            geoPolyline.vertices.add(currentCoordinates);
//                            mapTravelledPolylines.get(0).setGeometry(geoPolyline);
//                        }
//                    }, 350
//            );
//        }
//    }

    public void drawTrafficPolylineForRoute(Route route, boolean isTrafficOn) {
        isTrafficEnabled = isTrafficOn;
        new DrawTrafficLines().execute(new ArrayList<>(route.getSections()));
//        if (isTrafficOn) {
//            new Handler().postDelayed(() -> {
//                clearMapTrafficPolyline();
////                for (Section section : route.getSections()) {
//                if(!route.getSections().isEmpty()){
//                    Section section = route.getSections().get(0);
//                    if(section.getSpans().size() > 50000) {
//                        int dividedValue = section.getSpans().size()/50000;
//                        for (int i = 0; i < dividedValue; i++) {
//                            if(i<dividedValue-1){
//                                Utils.calculateSpans(context, 50000*i, (50000*i)+50000, section);
//                            }else{
//                                Utils.calculateSpans(context, 50000*i, section.getSpans().size(), section);
//                            }
//                        }
//                    }else{
//                        Utils.calculateSpans(context, 0, section.getSpans().size(), section);
//                    }
//                }
//                //draw traffic polyline map
//                ((Activity) context).runOnUiThread(() -> {
//                    try {
//                        for (TrafficLineColor trafficLine : trafficLineColors) {
//                            GeoPolyline trafficGeoPolyline = new GeoPolyline(trafficLine.getCoordinatesList());
//                            MapPolyline trafficRouteMapPolyline = new MapPolyline(trafficGeoPolyline, 20.0, trafficLine.getLineColor());
//                            trafficRouteMapPolyline.setDrawOrder(1);
//                            trafficRouteMapPolyline.setOutlineColor(trafficLine.getLineColor());
//                            trafficRouteMapPolyline.setOutlineWidth(5.0);
//                            trafficPolylines.add(trafficRouteMapPolyline);
//                            //add polyline to map view
//                            mapView.getMapScene().addMapPolyline(trafficRouteMapPolyline);
//                        }
//                    } catch (InstantiationErrorException e) {
//                        e.printStackTrace();
//                    }
//                    LoadingDialog.dismissDialog(dialog);
//                });
//            }, 100);
//        }
    }

    public void clearMap() {
        clearWaypointMapMarker();
        clearRoute();

        navigationExample.stopNavigation();
    }

    private void clearWaypointMapMarker() {
        for (MapMarker mapMarker : mapMarkerList) {
            mapView.getMapScene().removeMapMarker(mapMarker);
        }
        mapMarkerList.clear();
    }

    private void clearRoute() {
        for (MapPolyline mapPolyline : mapPolylines) {
            mapView.getMapScene().removeMapPolyline(mapPolyline);
        }
        mapPolylines.clear();
//        //travelled path
//        for (MapPolyline mapPolyline : mapTravelledPolylines) {
//            mapView.getMapScene().removeMapPolyline(mapPolyline);
//        }
//        mapTravelledPolylines.clear();
    }

    private void clearMapTrafficPolyline() {
        if (trafficPolylines != null) {
            for (MapPolyline polyline : trafficPolylines) {
                mapView.getMapScene().removeMapPolyline(polyline);
            }

            trafficPolylines.clear();
            trafficPolylines = null;
        }
    }

    private void setLongPressGestureHandler() {
        mapView.getGestures().setLongPressListener((gestureState, touchPoint) -> {
            GeoCoordinates geoCoordinates = mapView.viewToGeoCoordinates(touchPoint);
            if (geoCoordinates == null) {
                return;
            }
            if (gestureState == GestureState.BEGIN) {
                clearWaypointMapMarker();
                clearRoute();
                destinationWaypoint = new Waypoint(geoCoordinates);
                addCircleMapMarker(geoCoordinates, R.drawable.green_dot);
                isLongpressDestination = true;
                messageView.setText("New long press destination set.");
            }
        });
    }

    private GeoCoordinates createRandomGeoCoordinatesAroundMapCenter() {
        GeoCoordinates centerGeoCoordinates = getMapViewCenter();
        double lat = centerGeoCoordinates.latitude;
        double lon = centerGeoCoordinates.longitude;
        return new GeoCoordinates(getRandom(lat - 0.02, lat + 0.02),
                getRandom(lon - 0.02, lon + 0.02));
    }

    private double getRandom(double min, double max) {
        return min + Math.random() * (max - min);
    }

    private GeoCoordinates getMapViewCenter() {
        return mapView.getCamera().getState().targetCoordinates;
    }

    private void addCircleMapMarker(GeoCoordinates geoCoordinates, int resourceId) {
        MapImage mapImage = MapImageFactory.fromResource(context.getResources(), resourceId);
        MapMarker mapMarker = new MapMarker(geoCoordinates, mapImage);

        mapView.getMapScene().addMapMarker(mapMarker);
        mapMarkerList.add(mapMarker);
    }

    private void showDialog(String title, String message) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle(title)
                .setMessage(message)
                .show();
    }


    private void showBetterRouteDialog(Route route, int etaDifferenceInSeconds) {
        AlertDialog.Builder builder = new AlertDialog.Builder(context);
        builder.setTitle("Alert!")
                .setMessage("A better route is found for your journey, which can save your " + Utils.makeDurationString(etaDifferenceInSeconds) + ". Do you want to switch to that route?")
                .setPositiveButton("Yes", (dialogInterface, i) -> {
                    animateToRouteStart(route);
                }).setNegativeButton("No", (dialogInterface, i) -> {
                    dialogInterface.dismiss();
                })
                .show();
    }

//    public void showStartNavigationDialog(String title, String message, Route route, boolean isSimulated) {
//
//        String simulatedText = "Start navigation (simulated)";
//        String deviceLocText = "Start navigation (device location)";
//        AlertDialog.Builder builder = new AlertDialog.Builder(context);
//        builder.setTitle(title)
//                .setMessage(message)
//                .setPositiveButton(simulatedText,
//                        (dialog, which) -> {
//                            navigationExample.startNavigation(route, true);
//                        })
//                .setNeutralButton(deviceLocText,
//                        (dialog, which) -> {
//                            navigationExample.startNavigation(route, false);
//                        })
//                .show();
//        EventBus.getDefault().post(new ShowNavigationDialog());
//    }

    public void switchVoiceEnables(boolean voiceEnabled) {
        navigationExample.switchVoiceEnables(voiceEnabled);
    }

    public boolean isVoiceEnabled() {
        return navigationExample.isVoiceEnabled();
    }

    public void stopLocating() {
        navigationExample.stopLocating();
    }

    @Override
    public void onUpdateRoute(Route route, RouteProgress routeProgress) {
//        if (isSimulated) {
//            startWaypoint = new Waypoint(getMapViewCenter());
//        } else {
//            startWaypoint = new Waypoint(navigationExample.getLastKnownLocation().coordinates);
//        }
//        calculateRouteRefresh(isSimulated, selectedOption,isTrafficEnabled);
//        //Toast.makeText(context, "onUpdateRoute", Toast.LENGTH_SHORT).show();
//        clearRoute();
//        if (navigationExample.getLastKnownLocation() == null) {
//            showDialog("Error", "No location found.");
//            return;
//        }
//        Location location = navigationExample.getLastKnownLocation();
////            if(startWaypoint==null)
//        startWaypoint = new Waypoint(location.coordinates);
//        // If a driver is moving, the bearing value can help to improve the route calculation.
//        startWaypoint.headingInDegrees = location.bearingInDegrees;
//        mapView.getCamera().lookAt(location.coordinates);
//        routeCalculator.calculateRoute(startWaypoint, destinationWaypoint, selectedOption, (routingError, routes) -> {
//            if (routingError == null && routes != null) {
//                Route newRoute = routes.get(0);
//                showRouteOnMap(newRoute);
//            } else {
//                showDialog("Error while calculating a route:", routingError.toString());
//            }
//        });
//          showRouteOnMap(route);
        updateRouteProgress(route, routeProgress);
    }

    @Override
    public void onFoundBetterRoute(Route route, int etaDifferenceInSeconds) {
        showBetterRouteDialog(route, etaDifferenceInSeconds);
    }

    @Override
    public void onUpdateRoute(NavigableLocation currentNavigableLocation, VisualNavigator visualNavigator) {
//        showRouteOnMapRefresh(currentNavigableLocation.mapMatchedLocation != null ? currentNavigableLocation.mapMatchedLocation.coordinates : currentNavigableLocation.originalLocation.coordinates, visualNavigator);
    }

    @Override
    public void onRouteDeviation(Route route) {
        calculateRoute(isSimulated, selectedOption, null, false);
    }

    // Animate to custom guidance perspective, centered on start location of route.
    public void animateToRouteStart(Route route) {
        // The first coordinate marks the start location of the route.
//        GeoCoordinates startOfRoute = mapView.getCamera().getState().targetCoordinates;//route.getGeometry().vertices.get(0);
        GeoCoordinates startOfRoute = route.getGeometry().vertices.get(0);
        GeoCoordinatesUpdate geoCoordinatesUpdate = new GeoCoordinatesUpdate(startOfRoute);

        Double bearingInDegrees = null;
        double tiltInDegrees = 70;
        GeoOrientationUpdate orientationUpdate = new GeoOrientationUpdate(bearingInDegrees, tiltInDegrees);

        double distanceInMeters = 75;
        MapMeasure mapMeasureZoom = new MapMeasure(MapMeasure.Kind.DISTANCE, distanceInMeters);

        double bowFactor = 1;
        MapCameraAnimation animation = MapCameraAnimationFactory.flyTo(
                geoCoordinatesUpdate, orientationUpdate, mapMeasureZoom, bowFactor, Duration.ofSeconds(4));
        mapView.getCamera().startAnimation(animation, new AnimationListener() {
            @Override
            public void onAnimationStateChanged(@NonNull AnimationState animationState) {
                if (animationState == AnimationState.COMPLETED || animationState == AnimationState.CANCELLED) {
                    toggleTrackingButtonOnClicked();
                    showRouteOnMap(route, isTrafficEnabled);
                    showRouteDetails(route, isSimulated);
                }
            }
        });
        //save route for saving in GPX
        routeCalculator.setCalculatedRoute(route);
    }

    class DrawTrafficLines extends AsyncTask<ArrayList<Section>, Void, Void> {

        @Override
        protected void onPreExecute() {
            if (!dialog.isShowing()) {
                LoadingDialog.showDialog(dialog);
            }
            super.onPreExecute();
        }

        @SafeVarargs
        @Override
        protected final Void doInBackground(ArrayList<Section>... arrayLists) {
            System.runFinalization();
            Runtime.getRuntime().runFinalization();
            Runtime.getRuntime().gc();
            System.gc();

//            List<Section> sectionList = new ArrayList<>(arrayLists[0]);
            clearMapTrafficPolyline();
            trafficPolylines = null;

            if (isTrafficEnabled) {
                if (!arrayLists[0].isEmpty()) {
                    List<TrafficLineColor> trafficLineColors = new ArrayList<>();
                    for (Section section : arrayLists[0]) {
                        trafficLineColors.addAll(Utils.calculateSpans(context, section.getSpans()));
                    }
//                    List<TrafficLineColor> trafficLineColors = Utils.calculateSpans(context, arrayLists[0].get(0).getSpans());
                    try {
                        trafficPolylines = new ArrayList<>();
                        for (TrafficLineColor trafficLine : trafficLineColors) {
                            GeoPolyline trafficGeoPolyline = new GeoPolyline(trafficLine.getCoordinatesList());
                            MapPolyline trafficRouteMapPolyline = new MapPolyline(trafficGeoPolyline, new MapPolyline.SolidRepresentation(new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 10), trafficLine.getLineColor(), new MapMeasureDependentRenderSize(RenderSize.Unit.PIXELS, 1), trafficLine.getLineColor(), LineCap.ROUND));
                            trafficRouteMapPolyline.setDrawOrder(1);
                            trafficPolylines.add(trafficRouteMapPolyline);
                            //add polyline to map view
                            mapView.getMapScene().addMapPolyline(trafficRouteMapPolyline);
                        }
                    } catch (InstantiationErrorException |
                             MapPolyline.Representation.InstantiationException |
                             MapMeasureDependentRenderSize.InstantiationException e) {
                        e.printStackTrace();
                    }
//                    arrayLists[0].clear();
                }
            }
            return null;
        }

        @Override
        protected void onPostExecute(Void unused) {
            LoadingDialog.dismissDialog(dialog);
            super.onPostExecute(unused);
        }
    }

}
