/*
 * Copyright (C) 2019-2022 HERE Europe B.V.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * SPDX-License-Identifier: Apache-2.0
 * License-Filename: LICENSE
 */

package com.example.heresdkandroidref.features;

import android.content.Context;
import android.net.ConnectivityManager;
import android.util.Log;

import androidx.annotation.Nullable;

import com.example.heresdkandroidref.utils.SharedPref;
import com.here.sdk.core.errors.InstantiationErrorException;
import com.here.sdk.routing.CalculateRouteCallback;
import com.here.sdk.routing.CarOptions;
import com.here.sdk.routing.OfflineRoutingEngine;
import com.here.sdk.routing.PedestrianOptions;
import com.here.sdk.routing.Route;
import com.here.sdk.routing.RoutingEngine;
import com.here.sdk.routing.RoutingInterface;
import com.here.sdk.routing.ScooterOptions;
import com.here.sdk.routing.TruckOptions;
import com.here.sdk.routing.Waypoint;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

// A class that creates car Routes with the HERE SDK.
public class RouteCalculator {
    private final Context context;
    private RoutingInterface routingEngine;
    private final RoutingEngine onlineRoutingEngine;
    private final OfflineRoutingEngine offlineRoutingEngine;
    @Nullable
    private Route calculatedRoute;
    public RouteCalculator(Context context) {
        this.context = context;
//        if (isDeviceConnected()) {
            try {
                onlineRoutingEngine = new RoutingEngine();
            } catch (InstantiationErrorException e) {
                throw new RuntimeException("Initialization of RoutingEngine failed: " + e.error.name());
            }
//        } else {
            try {
                // Allows to calculate routes on already downloaded or cached map data.
                // For downloading offline maps, please check the OfflineMaps example app.
                // This app uses only cached map data that gets downloaded when the user
                // pans the map. Please note that the OfflineRoutingEngine may not be able
                // to calculate a route, when not all map tiles are loaded. Especially, the
                // vector tiles for lower zoom levels are required to find possible paths.
                offlineRoutingEngine = new OfflineRoutingEngine();
            } catch (InstantiationErrorException e) {
                throw new RuntimeException("Initialization of OfflineRoutingEngine failed: " + e.error.name());
            }
//        }
        setRoutingEngine();
    }

    private void setRoutingEngine() {
        if (isDeviceConnected()) {
            routingEngine = onlineRoutingEngine;
        } else {
            routingEngine = offlineRoutingEngine;
        }
    }

    public void calculateRoute(Waypoint startWaypoint,
                               List<Waypoint> waypointsList,
                               Waypoint destinationWaypoint,
                               String selectedOption,
                               CalculateRouteCallback calculateRouteCallback) {
        setRoutingEngine();
        List<Waypoint> waypoints = new ArrayList<>();
        waypoints.add(startWaypoint);
        if(!waypointsList.isEmpty()){
            waypoints.addAll(waypointsList);
        }
        waypoints.add(destinationWaypoint);
        Log.e("sachin", selectedOption);
        // A route handle is required for the DynamicRoutingEngine to get updates on traffic-optimized routes.
        switch (selectedOption) {
            case "car":
//                CarOptions carOptions = new CarOptions();
                CarOptions carOptions = SharedPref.getInstance().getCarPreferenceData();
                carOptions.routeOptions.enableRouteHandle = true;
                routingEngine.calculateRoute(
                        waypoints,
                        carOptions,
                        calculateRouteCallback);
                break;
            case "truck":
//                TruckOptions truckOptions = new TruckOptions();
                TruckOptions truckOptions = SharedPref.getInstance().getTruckPreferenceData();
                truckOptions.routeOptions.enableRouteHandle = true;
                routingEngine.calculateRoute(
                        waypoints,
                        truckOptions,
                        calculateRouteCallback);
                break;
            case "scooter":
//                ScooterOptions scooterOptions = new ScooterOptions();
                ScooterOptions scooterOptions = SharedPref.getInstance().getScooterPreferenceData();
                scooterOptions.routeOptions.enableRouteHandle = true;
                routingEngine.calculateRoute(
                        waypoints,
                        scooterOptions,
                        calculateRouteCallback);
                break;
            case "walk":
//                PedestrianOptions pedestrianOptions = new PedestrianOptions();
                PedestrianOptions pedestrianOptions = SharedPref.getInstance().getWalkPreferenceData();
                pedestrianOptions.routeOptions.enableRouteHandle = true;
                routingEngine.calculateRoute(
                        waypoints,
                        pedestrianOptions,
                        calculateRouteCallback);
                break;
        }

//        CarOptions carOptions = new CarOptions();
//        carOptions.routeOptions.enableRouteHandle = true;
//        routingEngine.calculateRoute(
//                waypoints,
//                carOptions,
//                calculateRouteCallback);
    }

    private boolean isDeviceConnected() {
        // An application may define here a logic to determine whether a device is connected or not.
        // For this example app, the flag is set from UI.
//        return isDeviceConnected && !isOfflineModeEnabled;
        ConnectivityManager connectivityManager = ((ConnectivityManager) context.getSystemService(Context.CONNECTIVITY_SERVICE));
        boolean isDeviceConnected = connectivityManager.getActiveNetworkInfo() != null && connectivityManager.getActiveNetworkInfo().isConnected();
        return isDeviceConnected && !SharedPref.getInstance().getOfflineMode();
    }

    public Route getCalculatedRoute() {
        if (calculatedRoute == null) {
            throw new RuntimeException("No route calculated yet.");
        }

        return calculatedRoute;
    }

    public void setCalculatedRoute(Route route) {
        if (route != null) {
            calculatedRoute = route;
        }
    }
}
